import polars as pl
import pandas as pd
from .path_handler import *
from .file_handler import *
from .df_handler import *
from .arg_parser import *

def pick_rows_contain_certain_value(path, col_to_apply_on, value, result_path="result.parquet"):
    df = parse_args(path, col_to_apply_on, value, result_path)

    if is_single_file(path):
        df = df_pick_rows_contain_certain_value_on_col(df, col_to_apply_on, value)
        output_file_from_df(df, result_path)
    else:
        with zipfile.ZipFile(result_path,'a') as output_file:
            for name,d in df.items():
                d = df_pick_rows_contain_certain_value_on_col(d, col_to_apply_on, value)
                output_file.writestr(name, output_str_from_df(d, name))

def turn_vertical_to_horizontal(path, col_to_apply_on, value, result_path="result.parquet"):
    df = parse_args(path, col_to_apply_on, value, result_path)

    if is_single_file(path):
        df = df_turn_vertical_to_horizontal(df, col_to_apply_on, value)
        output_file_from_df(df, result_path)
    else:
        with zipfile.ZipFile(result_path,'a') as output_file:
            for name,d in df.items():
                d = df_turn_vertical_to_horizontal(d, col_to_apply_on, value)
                output_file.writestr(name, output_str_from_df(d, name))

if __name__ == '__main__':
    n = input('選擇想要的功能 輸入1:挑選出指定欄位為某值的所有列 輸入2:將直欄轉換為橫向資料 輸入0:離開\n')
    while(n!='0'):
        if(n=='1'):
            print('已選功能1:挑選出指定欄位為某值的所有列')
            path, col_to_apply_on, value, result_path = input('以空白隔開 請依序輸入:要處理的檔名(路徑/url) 指定值所在的欄位 指定的值 輸出的檔名(路徑)\n').split(' ')
            pick_rows_contain_certain_value(path = path, col_to_apply_on = col_to_apply_on, value = value, result_path = result_path)
        elif(n=='2'):
            print('已選功能2:將直欄轉換為橫向資料')
            path, col_to_apply_on, value, result_path = input('以空白隔開 請依序輸入:要處理的檔名(路徑/url) 轉換為欄位標題的欄位 對應值所在的欄位 輸出的檔名(路徑)\n').split(' ')
            turn_vertical_to_horizontal(path = path, col_to_apply_on = col_to_apply_on, value = value, result_path = result_path)
        n = input('選擇想要的功能 輸入1:挑選出指定欄位為某值的所有列 輸入2:將直欄轉換為橫向資料 輸入0:離開\n')


import pandas as pd
import polars as pl
from .path_handler import *
from .file_handler import *
from .df_handler import *

_supported_types_ = [".parquet", ".csv", ".csv.gz", ".zip",]

def parse_args(path, col_to_apply_on, value, result_path):
    if type(path) is pd.DataFrame() or pl.DataFrame():
        df = pl.DataFrame(path)
    elif not is_url(path) and not file_exists(path):
        raise ValueError(f'{path} does not exist or it is a directory (not file)')
    elif ext_str(result_path) not in _supported_types_:
        raise ValueError(f'output file type: {ext_str(result_path)} not supported yet, supported types {_supported_types_}')
    else:
        df = df_readfile(path)
    if df is None:
        raise ValueError(f'input file type: {ext_str(path)} not supported yet, supported types {_supported_types_}')
    pathlib.Path.mkdir(parent(result_path), exist_ok=True, parents=True)
    return df



'''
def int_df_read_csv_gz_from_url(url):
    csv_path = f'{os.getcwd()}/{os.path.split(url)[1]}'
    while(True):
        try:
            content = requests.get(url).content
        except requests.exceptions.RequestException:
            print('\nhttp request error, retry...')
            continue
        break    
    open(csv_path, "wb").write(content)
    try:
        df = pd.read_csv(csv_path, compression='gzip', header=0, sep=',', quotechar='"')
    except:
        #print('[no file found]', end=' ')
        df = None
        os.remove(csv_path)
        return 0,df
    size = os.path.getsize(csv_path)
    os.remove(csv_path)
    return size, df
'''


import polars as pl
import pandas as pd
from .path_handler import *
import zipfile
import zlib
import gzip
import requests
import os

def name_df_dict_read_zip(path):
    d = dict()
    zip_info_list = zipfile.ZipFile(path).infolist()
    for f in zip_info_list:
        df = df_readfile(f.filename, zipfile.ZipFile(path).read(name=f))
        if df:
            d[f.filename] = df
    return d

def df_readfile(path, f=None):
    if not f:
        f = path
    if is_url(path):
        dl_file_path = f'tmp/{pathlib.Path(path).name}'
        pathlib.Path('tmp').mkdir(parents=True, exist_ok=True)
        content = requests.get(path).content
        open(dl_file_path, "wb").write(content)
        df_out  = df_readfile(dl_file_path)
        os.remove(dl_file_path)
        return df_out
    elif is_csv(path):
        return pl.read_csv(f)
    elif is_csv_gz(path):
        return pl.read_csv(f, gzip=True)
    elif is_parquet(path):
        return pl.read_parquet(f)
    elif is_zip(path):
        return name_df_dict_read_zip(path)
    else:
        raise ValueError(f'Invalid file format: {ext_str(f)}')

def output_file_from_df(df, path):
    if is_csv(path):
        df.to_csv(path)
    elif is_csv_gz(path):
        df.to_csv(path, gzip=True)
    elif is_parquet(path):
        df.to_parquet(path)
    else:
        raise ValueError(f'Invalid file format: {ext_str(path)}')

def output_str_from_df(df, path):
    if is_csv(path):
        return df.to_csv()
    elif is_csv_gz(path):
        s = df.to_csv()
        b = bytes(f'\ufeff{s}', 'utf-8')
        return gzip.compress(b)
    
    elif is_parquet(path):
        return df.to_parquet()
        pl.DataFrame().to_csv()
    else:
        raise ValueError(f'Invalid file format: {ext_str(path)}')

if __name__ == "__main__":
    path = 'test.csv.gz'
    d = df_readfile(path)
    if not is_zip(path):
        #print(d.head())
        print(d.to_csv())
    else:
        for name, df in d.items():
            print(name, df.head())


import datetime as dt
import psutil
import os

def float_memory_usage():
    return round(psutil.Process(os.getpid()).memory_info().rss/1000000,2)

def dt_sec_timer():
    '''usage:
    timer = dt_sec_timer()
    'next(timer)' will return/yield the time since last call of 'next(timer)'
    if it's first to be called, it will return/yield 0.00
    '''
    start_time = dt.datetime.now()
    while True:
        end_time = dt.datetime.now()
        last_exec = (end_time - start_time).total_seconds()
        start_time = dt.datetime.now()
        yield last_exec

def df_drop_columns_except(df, exception):
    df_out = df.loc[:, df.columns.isin(exception)]
    return df_out
    
def sec_to_HMS(sec):
    return '{H:0>2d}:{M:0>2d}:{S:0>2d}'.format(H=int(int(sec)/3600), M=int(int(sec)%3600/60), S=int(sec)%60)
    
def list_listfile(mypath):
    onlyfiles = [f for f in listdir(mypath) if isfile(join(mypath, f))]
    return onlyfiles




from cameo_table import *

def main():
    n = input('選擇想要的功能 輸入1:挑選出指定欄位為某值的所有列 輸入2:將直欄轉換為橫向資料 輸入0:離開\n')
    while(n!='0'):
        if(n=='1'):
            print('已選功能1:挑選出指定欄位為某值的所有列')
            path, col_to_apply_on, value, result_path = input('以空白隔開 請依序輸入:要處理的檔名(路徑/url) 指定值所在的欄位 指定的值 輸出的檔名(路徑)\n').split(' ')
            try:
                pick_rows_contain_certain_value(path = path, col_to_apply_on = col_to_apply_on, value = value, result_path = result_path)
            except Exception as e:
                print(e)
        elif(n=='2'):
            print('已選功能2:將直欄轉換為橫向資料')
            path, col_to_apply_on, value, result_path = input('以空白隔開 請依序輸入:要處理的檔名(路徑/url) 轉換為欄位標題的欄位 對應值所在的欄位 輸出的檔名(路徑)\n').split(' ')
            try:
                turn_vertical_to_horizontal(path = path, col_to_apply_on = col_to_apply_on, value = value, result_path = result_path)
            except Exception as e:
                print(e)
        n = input('選擇想要的功能 輸入1:挑選出指定欄位為某值的所有列 輸入2:將直欄轉換為橫向資料 輸入0:離開\n')

if __name__ == '__main__':
    main()


import polars as pl
from .file_handler import *

def df_pick_rows_contain_certain_value_on_col(df_in, col_to_apply_on, value):
    df = df_in
    df = df[df[col_to_apply_on]==value]
    return df

def df_turn_vertical_to_horizontal_old(df_in, col_of_label, col_of_value):
    df = df_in
    dup_keys = df.columns
    if col_of_value not in dup_keys or col_of_label not in dup_keys:
        raise ValueError('column index not in dataframe')
    dup_keys.remove(col_of_label)
    dup_keys.remove(col_of_value)
    group_key = dup_keys[0]
    max_count = 0
    for key in dup_keys:
        g = df[dup_keys].groupby(key).n_unique().shape[0]
        if(g>max_count):
            max_count = g
            group_key = key
    indexes = list(df.groupby(col_of_label).groups()[:,0])
    groups = list(df.groupby(group_key).groups()[:,0])
    df_out = pl.DataFrame()
    for group in groups:
        new_row = df[df[group_key]==group][dup_keys][0]
        for index in indexes:
            try:
                tmp = df[df[group_key]==group]
                tmp = tmp[tmp[col_of_label]==index]
                new_row[index] = tmp[col_of_value]
            except Exception as e:
                new_row[index] = [float("NaN")]
        df_out.vstack(new_row, True)
    return df_out

def df_turn_vertical_to_horizontal(df_in, col_of_label, col_of_value):
    # bug: pivot index can not be float
    df_out = df_in
    dup_keys = df_out.columns
    dup_keys.remove(col_of_label)
    dup_keys.remove(col_of_value)
    group_key = dup_keys[0]
    max_count = 0
    for key in dup_keys:
        g = df_out[dup_keys].groupby(key).n_unique().shape[0]
        if(g>max_count):
            max_count = g
            group_key = key
    df_out = df_out.pivot(index = group_key, columns = col_of_label, values = col_of_value)
    df_left = df_in.drop(col_of_label).drop(col_of_value)
    df_left = df_left.distinct(subset=dup_keys)
    df_out = df_out.drop(group_key)
    df_out = df_left.hstack(df_out)
    return df_out

def unit_test_pivot():
    d = {
            "foo":["one", "one", "one", "two", "two", "two"],
            "bar":["A", "B", "C", "A", "B", "C"],
            "baz":[2.0, 2.0, 2.0, 2.0, 2.0, 2.0],
            "value":[11, 12, 13, 14, 15, 16]
        }
    df = pl.DataFrame(d)
    df = df.pivot(index=["foo", "baz"], columns="bar", values="value")
    (df)

if __name__ == '__main__':
    '''
    dfs = df_readfile('test.zip')
    for zipname, df in dfs.items():
        df_turn_vertical_to_horizontal(df, 'sensorId', 'value')
        df_pick_rows_contain_certain_value_on_col(df, 'sensorId', 'temperature')
    '''
    df = df_readfile('deviceId_11094256953_original.csv')
    #print(df.head())
    df = df_turn_vertical_to_horizontal(df, 'sensorId', 'value')
    print(df.head())
    #unit_test_pivot()


import pathlib
import zipfile

def parent(path):
    return pathlib.Path(path).parent

def file_exists(path):
    return pathlib.Path(path).exists() and not pathlib.Path(path).is_dir()

def ext_parser(path):
    return pathlib.Path(path).suffixes

def ext_str(path):
    filetypelist = ext_parser(path)
    filetype = ''
    for ext in filetypelist:
        filetype += f'{ext}'
    return filetype

def is_url(path):
    return path[0:7].lower()=='http://' or path[0:8].lower()=='https://'

def is_zip(path):
    if type(path) is zipfile.ZipInfo:
        path = path.filename
    file_ext = ext_parser(path)
    return file_ext[-1] == '.zip'

def is_csv(path):
    if type(path) is zipfile.ZipInfo:
        path = path.filename
    file_ext = ext_parser(path)
    return file_ext[-1] == '.csv'

def is_parquet(path):
    if type(path) is zipfile.ZipInfo:
        path = path.filename
    file_ext = ext_parser(path)
    return file_ext[-1] == '.parquet'

def is_csv_gz(path):
    if type(path) is zipfile.ZipInfo:
        path = path.filename
    file_ext = ext_parser(path)
    return file_ext[-2:] == ['.csv', '.gz']

def is_single_file(path):
    return not is_zip(path)

def filename_without_ext(path):
    path_out = path
    while(pathlib.Path(path_out).suffixes):
        path_out = pathlib.Path(path_out).stem
    return path_out

if __name__ == '__main__':
    print(ext_str('path/to/file/abc.def.ghi.jkl'))
    print(ext_parser('HtTp://abc.def.ghi.jkl'))


from setuptools import setup

setup(
    name='cameo_table',
    version='0.2.1',
    description='modify table file, and save modified file result',
    long_description = open('README.md','r',encoding='utf-8').read(),
    long_description_content_type = 'text/markdown',
    release_notes='fixed bugs, added documentation',
    url='https://github.com/bohachu/cameo_table',
    author='JC Wang',
    author_email='jcxgtcw@gmail.com',
    license='BSD 2-clause',
    packages=['cameo_table'],
    install_requires=[
        'fastparquet==0.8.1',
        'numpy==1.22.3',
        'pandas==1.4.2',
        'polars==0.13.40',
        'pyarrow==7.0.0',
        'requests==2.28.0'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
    ],
)